/*
 * Copyright 2023 Red Hat, Inc.
 * Copyright 2022 Google LLC
 * SPDX-License-Identifier: MIT
 */

#ifndef ASAHI_PROTO_H_
#define ASAHI_PROTO_H_

/*
 * Possible cmd types for "command stream", ie. payload of EXECBUF ioctl:
 */
enum asahi_ccmd {
   ASAHI_CCMD_NOP = 1, /* No payload, can be used to sync with host */
   ASAHI_CCMD_IOCTL_SIMPLE,
   ASAHI_CCMD_GET_PARAMS,
   ASAHI_CCMD_GEM_NEW,
   ASAHI_CCMD_GEM_BIND,
   ASAHI_CCMD_SUBMIT,
   ASAHI_CCMD_SYNCOBJ_WAIT,
   ASAHI_CCMD_SYNCOBJ_CREATE,
};

#define ASAHI_CCMD(_cmd, _len)                                                           \
   (struct drm_ccmd_req)                                                                 \
   {                                                                                     \
      .cmd = ASAHI_CCMD_##_cmd, .len = (_len),                                           \
   }

/*
 * ASAHI_CCMD_NOP
 */
struct asahi_ccmd_nop_req {
   struct drm_ccmd_req hdr;
};

/*
 * ASAHI_CCMD_IOCTL_SIMPLE
 *
 * Forward simple/flat IOC_RW or IOC_W ioctls.  Limited ioctls are supported.
 */
struct asahi_ccmd_ioctl_simple_req {
   struct drm_ccmd_req hdr;

   uint32_t cmd;
   uint8_t payload[];
};
DEFINE_CAST(drm_ccmd_req, asahi_ccmd_ioctl_simple_req)

struct asahi_ccmd_ioctl_simple_rsp {
   struct drm_ccmd_rsp hdr;

   /* ioctl return value, interrupted syscalls are handled on the host without
    * returning to the guest.
    */
   int32_t ret;

   /* The output payload for IOC_RW ioctls, the payload is the same size as
    * asahi_context_cmd_ioctl_simple_req.
    *
    * For IOC_W ioctls (userspace writes, kernel reads) this is zero length.
    */
   uint8_t payload[];
};

struct asahi_ccmd_get_params_req {
   struct drm_ccmd_req hdr;
   struct drm_asahi_get_params params;
};
DEFINE_CAST(drm_ccmd_req, asahi_ccmd_get_params_req)

struct asahi_ccmd_get_params_rsp {
   struct drm_ccmd_rsp hdr;
   int32_t ret;
   struct drm_asahi_params_global params;
};

struct asahi_ccmd_gem_new_req {
   struct drm_ccmd_req hdr;
   uint32_t flags;
   uint32_t bind_flags;
   uint32_t vm_id;
   uint32_t blob_id;
   uint64_t size;
   uint64_t addr;
};
DEFINE_CAST(drm_ccmd_req, asahi_ccmd_gem_new_req)

struct asahi_ccmd_gem_bind_req {
   struct drm_ccmd_req hdr;
   uint32_t op;
   uint32_t flags;
   uint32_t vm_id;
   uint32_t res_id;
   uint64_t size;
   uint64_t addr;
};
DEFINE_CAST(drm_ccmd_req, asahi_ccmd_gem_bind_req)

struct asahi_ccmd_gem_bind_rsp {
   struct drm_ccmd_rsp hdr;
   int32_t ret;
};

struct asahi_ccmd_submit_req {
   struct drm_ccmd_req hdr;
   uint32_t cmd_type;
   uint32_t flags;
   uint64_t cmdbuf_size;
   uint64_t result_offset;
   uint64_t result_size;
   uint32_t barriers[2];
   uint32_t queue_id;
   uint32_t result_handle;
   uint32_t in_sync_count;
   uint32_t out_sync_count;

   uint8_t payload[];
};
DEFINE_CAST(drm_ccmd_req, asahi_ccmd_submit_req)

struct asahi_ccmd_syncobj_wait_req {
   struct drm_ccmd_req hdr;
   uint32_t flags;
   uint32_t handle_count;
   int64_t timeout_nsecs;

   uint8_t payload[];
};
DEFINE_CAST(drm_ccmd_req, asahi_ccmd_syncobj_wait_req)

struct asahi_ccmd_syncobj_wait_rsp {
   struct drm_ccmd_rsp hdr;
   int32_t ret;
   uint32_t first_signaled;
};

struct asahi_ccmd_syncobj_create_req {
   struct drm_ccmd_req hdr;
   uint32_t flags;
};
DEFINE_CAST(drm_ccmd_req, asahi_ccmd_syncobj_create_req)

struct asahi_ccmd_syncobj_create_rsp {
   struct drm_ccmd_rsp hdr;
   int32_t ret;
   uint32_t handle;
};

#endif // ASAHI_PROTO_H_
